@@ -5,8 +5,10 @@ from dask import delayed
 from distributed import Client
 from distributed.client import futures_of
 from distributed.metrics import time
-from distributed.utils_test import gen_cluster, inc
-from distributed.utils_test import cluster, loop
+from distributed.utils_test import gen_cluster, inc, cluster
+from distributed.utils_test import loop # flake8: noqa
+from tornado import gen
+
 
 @gen_cluster(client=False)
 def test_publish_simple(s, a, b):
@@ -29,8 +31,8 @@ def test_publish_simple(s, a, b):
     result = yield f.scheduler.publish_list()
     assert result == ['data']
 
-    yield c.shutdown()
-    yield f.shutdown()
+    yield c.close()
+    yield f.close()
 
 
 @gen_cluster(client=False)
@@ -54,8 +56,8 @@ def test_publish_roundtrip(s, a, b):
     assert "not found" in str(exc_info.value)
     assert "nonexistent" in str(exc_info.value)
 
-    yield c.shutdown()
-    yield f.shutdown()
+    yield c.close()
+    yield f.close()
 
 
 @gen_cluster(client=True)
@@ -81,6 +83,7 @@ def test_unpublish(c, s, a, b):
     assert "not found" in str(exc_info.value)
     assert "data" in str(exc_info.value)
 
+
 def test_unpublish_sync(loop):
     with cluster() as (s, [a, b]):
         with Client(s['address']) as c:
@@ -155,5 +158,5 @@ def test_publish_bag(s, a, b):
 
     out = yield f.compute(result)
     assert out == [0, 1, 2]
-    yield c.shutdown()
-    yield f.shutdown()
+    yield c.close()
+    yield f.close()
